unit Obstacles;

interface

uses
  Classes, IniFiles, Forms, SysUtils, Dialogs;

type
  TVertex = Class (TObject)
  private
    Fx,Fy: integer;
  public
    property x:integer read Fx write Fx;
    property y:integer read Fy write Fy;
  end;

  TPolygon = Class (TObject)
  private
    fVertices: TList;                           // seznam vrcholu polygonu
  public
    constructor Create;
    destructor Destroy;override;
    property Vertices:TList read FVertices write FVertices;
  end;

  TObstacles = Class (TObject)
  private
    FObstacles:TList;                           // seznam prekazek (polygonu)
  public
    constructor Create;
    destructor Destroy;override;
    procedure Clean;
    procedure Save(FileName:string);            // ulozeni prekazek
    function Load(FileName:string):integer;     // nacteni prekazek
    property Obstacles:TList read FObstacles;
  end;

implementation

constructor TPolygon.Create;
begin
  inherited create;
  fVertices:=TList.Create;
end;

destructor TPolygon.Destroy;
var i: integer;
begin
  for i:=0 to fVertices.Count-1 do
    TVertex(fVertices.Items[i]).Free;
  fVertices.Free;
  inherited destroy;
end;

constructor TObstacles.Create;
begin
  inherited create;
  FObstacles:=TList.Create;
end;

destructor TObstacles.Destroy;
begin
  Clean;
  FObstacles.Free;
  inherited destroy;
end;

procedure TObstacles.Clean;
var i: integer;
begin
  for i:=0 to FObstacles.Count-1 do TPolygon(FObstacles.Items[i]).Free;
  FObstacles.Clear;
end;

procedure TObstacles.Save(FileName: string);
var DataFile: TIniFile;
  i,j,x,y: integer;
begin
  DataFile := TIniFile.Create(FileName);
  DataFile.UpdateFile;
  for i:=0 to fObstacles.Count-1 do
    begin
      for j:=0 to TPolygon(fObstacles.Items[i]).fVertices.Count-1 do
        begin
          x:=TVertex(TPolygon(fObstacles.Items[i]).fVertices[j]).Fx;
          y:=TVertex(TPolygon(fObstacles.Items[i]).fVertices[j]).Fy;
          DataFile.WriteInteger(inttostr(i+1)+'.polygon','x'+inttostr(j+1),x);
          DataFile.WriteInteger(inttostr(i+1)+'.polygon','y'+inttostr(j+1),y);
        end;
    end;
  DataFile.Free;
end;

function TObstacles.Load(FileName: string): integer;
var DataFile: TIniFile;
  i,j,k: integer;
  x,y: double;
  Section: TStringList;             // list polygon
  Values: TStringList;              // list hodnot (souadnic vrchol polygonu)
  polygon: TPolygon;
  extension: string;
  v: TVertex;
  bool: boolean;
begin
  DataFile := TIniFile.Create(FileName);
  extension:=ExtractFileExt(DataFile.FileName);
  if not(extension='.obs') then begin
      MessageDlg('Neplatn soubor!',mtWarning,[mbOK],0);
      exit;
   end;
  Clean;
  Section:=TStringList.Create;
  Values:=TStringList.Create;
  try
    DataFile.ReadSections(Section);
  except
    MessageDlg('Chyba pi natn souboru!',mtWarning,[mbOK],0);
    result:=-1;
    exit;
  end;
  for i:=1 to Section.Count do  // pocet polygon
  begin
    polygon:=TPolygon.Create;
    DataFile.ReadSectionValues(inttostr(i)+'.polygon',Values);
    k:=Values.Count mod 2;
    bool:=DataFile.SectionExists(inttostr(i)+'.polygon');
    if (k<>0)or(bool=false) then
      begin
        MessageDlg('Nekonzistentn poet vrchol v souboru!',mtWarning,[mbOK],0);
        result:=-1;
        exit;
      end;
    k:=trunc(Values.Count/2);
    for j:=1 to k do   // k: pocet vrchol polygonu
      begin
        x:=DataFile.ReadFloat(inttostr(i)+'.polygon','x'+inttostr(j),-1.0);
        y:=DataFile.ReadFloat(inttostr(i)+'.polygon','y'+inttostr(j),-1.0);
        if (x=-1)or(y=-1) then
        begin
          showmessage('Chyba pi natn vrchol ze souboru!');
          result:=-1;
          exit;
        end;
        v:=TVertex.Create;
        v.x:=trunc(x); v.y:=trunc(y);
        polygon.Vertices.Add(v);
        if (j=k) then FObstacles.Add(polygon);
      end;
  end;
  DataFile.Free;
  result:=1;
end;

end.
