unit CollisionDetection;

interface

uses
  Polygon, Classes;

type
  TSegment = Class            // usecka polygonu
  V1,V2: vertex;
  end;

  TSegments = array[1..2] of TList;

  TCollisionDetection = Class (TObject)
  private
    FSegments:TSegments;      // pole seznam usecek (2polygony)
    function Intersection(aS1,aS2:TSegment):boolean;  // prunik 2usecek =true
    procedure InsertToSegList(aPolygon:TPolygon;aNumPoly:integer);     // vlozeni usecek polygonu do seznamu usecek
    procedure InsertLineToSegList(aV1,aV2:TVertex;aNumPoly:integer);
  public
    constructor Create;
    destructor Destroy;override;    // detekce kolizi mezi useckou a polygonem
    procedure Clean;
    function CollisionDetection(aV1,aV2:TVertex;aPolygon2:TPolygon):boolean;overload;
    function CollisionDetection(aP1,aP2:TPolygon):boolean;overload;
  end;


implementation

{ TCollisionDetection }        

constructor TCollisionDetection.Create;
begin
  inherited create;
end;


destructor TCollisionDetection.Destroy;
begin
  inherited destroy;
end;


function TCollisionDetection.Intersection(aS1, aS2: TSegment): boolean;
var x1,x2,x3,x4,y1,y2,y3,y4 : double;   // souradnice vrcholu
            cua,cub,j,ua,ub : double;   // citatel parametru ua,ub,jmenovatel
begin
  result:=false;

  x1:=aS1.V1.x;
  y1:=aS1.V1.y;       // usecka 1
  x2:=aS1.V2.x;
  y2:=aS1.V2.y;

  x3:=aS2.V1.x;
  y3:=aS2.V1.y;       // usecka 2
  x4:=aS2.V2.x;
  y4:=aS2.V2.y;

  cua:=(x4-x3)*(y1-y3)-(y4-y3)*(x1-x3);
  cub:=(x2-x1)*(y1-y3)-(y2-y1)*(x1-x3);
  j:=(y4-y3)*(x2-x1)-(x4-x3)*(y2-y1);

    // jsou usecky rovnobezne?
  if (cua=0) and (cub=0) and (j=0) then exit;

    // existuje prusecik?
  ua:=cua/j;
  ub:=cub/j;
  if (0<=ua)and(ua<=1) and (0<=ub)and(ub<=1) then
    result:=true;
end;

          // vlozi usecky polygonu do seznamu usecek
procedure TCollisionDetection.InsertToSegList(aPolygon: TPolygon;aNumPoly:integer);
var i,j,k: integer;             // aNumPoly - cislo polygonu (1 pro prvni, 2 pro druhy)
      seg: TSegment;
      SegList: TList;
      x,y: double;
begin
  SegList:=TList.Create;
  for i:=0 to aPolygon.VertexCount-1 do
    begin
      seg:=TSegment.Create;
      x:=TVertex(aPolygon.Vertices.Items[i]).x;
      y:=TVertex(aPolygon.Vertices.Items[i]).y;
      seg.V1.x:=x; seg.V1.y:=y;
        // pozn: posl.vrchol polygonu se spoji s prvnim
      k:=aPolygon.VertexCount-1;
      if (i=k) then j:=-1 else j:=i;
      x:=TVertex(aPolygon.Vertices.Items[j+1]).x;
      y:=TVertex(aPolygon.Vertices.Items[j+1]).y;
      seg.V2.x:=x; seg.V2.y:=y;
      SegList.Add(seg);
    end;
  FSegments[aNumPoly]:=SegList;
end;

          // vlozi usecku do seznamu usecek
procedure TCollisionDetection.InsertLineToSegList(aV1,aV2:TVertex;aNumPoly:integer);
var  seg: TSegment;
  SegList: TList;
begin
  SegList:=TList.Create;
  seg:=TSegment.Create;
  seg.V1.x:=aV1.x;
  seg.V1.y:=aV1.y;

  seg.V2.x:=aV2.x;
  seg.V2.y:=aV2.y;

  SegList.Add(seg);
  FSegments[aNumPoly]:=SegList;
end;


function TCollisionDetection.CollisionDetection(aV1, aV2: TVertex;
  aPolygon2: TPolygon): boolean;
var  i: integer;
 s1,s2: TSegment;
begin
  result:=false;

  InsertLineToSegList(aV1,aV2,1);   
  InsertToSegList(aPolygon2,2);

  for i:=0 to TList(FSegments[2]).Count-1 do
    begin
      if Intersection(TList(FSegments[1]).Items[0],TList(FSegments[2]).Items[i])
        then
          begin
            result:=true;    // usecka protina polygon
            Clean;
            exit;
          end;
    end;
  Clean;
end;

function TCollisionDetection.CollisionDetection(aP1,
  aP2: TPolygon): boolean;
var i,j: integer;
begin
  result:=false;

  InsertToSegList(aP1,1);
  InsertToSegList(aP2,2);

  for i:=0 to TList(FSegments[1]).Count-1 do
    begin
      for j:=0 to TList(FSegments[2]).Count-1 do
        if Intersection(TList(FSegments[1]).Items[i],TList(FSegments[2]).Items[j])
        then
          begin   // polygony se protinaji
            result:=true;
            Clean;
            exit;
          end;
    end;
  Clean;
end;

procedure TCollisionDetection.Clean;
var i: integer;
begin
  for i:=0 to TList(FSegments[1]).Count-1 do
    TObject(TList(FSegments[1]).Items[i]).Free;
  for i:=0 to TList(FSegments[2]).Count-1 do
    TObject(TList(FSegments[2]).Items[i]).Free;
  TList(FSegments[1]).Free;
  TList(FSegments[2]).Free;
  FSegments[1]:=nil; FSegments[2]:=nil;
end;

end.



