unit Graph;

interface

uses
  Classes, SysUtils, Math;

type
  TID = integer;
  TData = TObject;
  TEvaluation=integer;

  TNode=class(TObject)
  private
   FID: TID;
  protected
    FEvaluation: TEvaluation;
    FDepth: TID;
  public
   function GetNodeLabel:string;virtual;
   constructor create(aID:TID);
   property ID:TID read FID write FID;
   property Depth:TID read FDepth write FDepth;
   property Evaluation:TEvaluation read FEvaluation write FEvaluation;
  end;

  TEdge=class(TObject)
  private
   FFromID: TID;
   FToID: TID;
  public
   property FRomID:integer read FFRomID write FFromID;
   property ToID:integer read FToID write FToID;
   constructor create(aFromID,aToID:TID);
  end;

  TGraph=class(TObject)
  private
    FEdges:TList;            //vlastnik vsech instanci tridy TEdge
    FNodes:TList;            //vlastnik vsech instanci tridy TNode
    FSolution:TList;         //spravce vybranych TNode, ktere predstavuji reseni
    FSolEdges:TList;         //spravce vybranych TEdge

    function GetNode(aID:TID):TNode;
    function GetNodeIndex(aID:TID):integer;
  protected
    function RulesCount:integer;virtual;abstract;
    function IsEqualData(aNode1,aNode2:TNode):boolean;virtual;abstract;
  public
    constructor create;
    destructor destroy;override;
    procedure CleanAll;
    function AddNode(aNode:TNode):integer;
    function AddEdge(aEdge:TEdge):integer;
    function RebuildSolution:boolean;
    function getMaxNodeID:integer;
    function GetPrev(aNode:TNode):TNode;
    function GetEdge(aFrom,aTo:integer):TEdge;
    function IsIn(aList:TList;aNode:TNode):integer;
    function SortedList(aList:TList;aNode:TNode):integer; //setd seznam od nejv.(ohodnocen)
    property Node[NodeID:TID]:TNode read getNode;
    property Nodes:TList read FNodes;
    property Solution:TList read FSolution write FSolution;
    property SolEdges:TList read FSolEdges write FSolEdges;
  end;

implementation

{---------  // TNode // --------- }
constructor TNode.create(aID:TID);
begin
  inherited create;
  FID:=aID;
end;

{---------  // TEdge // --------- }
constructor TEdge.create(aFromID,aToID:TID);
begin
  inherited create;
  FFromID:=aFromID;
  FtoID:=aToID;
end;

{---------  // TGraph // --------- }
constructor TGraph.create;
begin
     inherited create;
     Fedges:=TList.Create;
     FNodes:=TList.Create;
     FSolution:=TList.Create;
     FSolEdges:=TList.Create;
end;

destructor TGraph.destroy;
begin
     CleanAll;
     FSolution.Free;
     FEdges.Free;
     FNodes.Free;
     inherited destroy;
end;

procedure TGraph.CleanAll;
var I:integer;
begin
     FSolution.Clear;
     FSolEdges.Clear;
     for I:=0 to (FEdges.Count-1) do
      begin
        TEdge(FEdges.Items[I]).Free;
      end;
     FEdges.Clear;
     for I:=0 to (FNodes.Count-1) do
      begin
        TNode(FNodes.Items[I]).Free;
      end;
     FNodes.Clear;
end;

function TGraph.AddNode(aNode:TNode):integer;
begin
     result:=FNodes.Add(aNode);
end;

function TGraph.AddEdge(aEdge:TEdge):integer;              
begin
     result:=FEdges.Add(aEdge);
end;

function TGraph.GetPrev(aNode:TNode):TNode;
var I:integer;
    goalID:TID;
begin
     result:=nil;
     goalID:=aNode.ID;
     for I:=0 to (FEdges.Count-1) do
      if TEdge(FEdges.Items[I]).ToID = goalID then result:=Node[TEdge(FEdges.Items[I]).FRomID];
end;

function TGraph.GetEdge(aFrom,aTo:integer):TEdge;
var i:integer;
    goalID:TID;
begin
    result:=nil;
    for i:=(FEdges.Count-1)downto 0 do
      if TEdge(FEdges.Items[i]).ToID = aTo then
        result:=TEdge(FEdges.Items[i]);
end;

function TGraph.RebuildSolution:boolean;
var pom: TNode;
    pom2: TNode;
    edge: TEdge;
    i: integer;
begin
     result:=false;
     if (FSolution=nil) or (FSolution.Count<>1) then exit;
     pom2 := TNode(FSolution.Items[0]);
     pom := GetPrev( TNode(FSolution.Items[0]) );

     for i:=(FEdges.Count-1) downto 0  do
      if (TEdge(FEdges.Items[i]).FFromID = pom.FID) and
       (TEdge(FEdges.Items[i]).FToID = pom2.FID) then
        begin
          edge:=FEdges.Items[i];
          break;
        end;

     while (pom<>nil) do
       begin
          FSolEdges.Insert(0,edge);
          FSolution.Insert(0,pom);
          pom2 := TNode(FSolution.Items[0]);
          pom := GetPrev( TNode(FSolution.Items[0]) );
          if (pom=nil) then break;
          for i:=(FEdges.Count-1) downto 0 do
            if (TEdge(FEdges.Items[i]).FFromID = pom.FID) and
            (TEdge(FEdges.Items[i]).FToID = pom2.FID) then
            begin
              edge:=FEdges.Items[i];
              break;
            end;
       end;
     result:=true;
end;

function TGraph.GetNodeIndex(aID:TID):integer;
var I:integer;
begin
     result:=-1;
     for I:=0 to FNodes.Count-1 do
        if TNode(FNodes.Items[I]).ID=aID then result:=I;
end;

function TGraph.GetNode(aID:TID):TNode;
var pom:integer;
begin
     result := nil;
     pom := GetNodeIndex(aID);
     if pom=-1 then exit;
     result:= FNodes.Items[ pom ]
end;

function TGraph.getMaxNodeID:integer;
var I:integer;
    max:TID;
begin
     result:=-1;
     if Fnodes = nil then exit;
     if FNodes.Count = 0 then exit; //vrati -1
     max:=0;
     for I:=0 to (FNodes.Count-1) do
        if (TNode(FNodes.Items[I]).ID>max) then max:=TNode(FNodes.Items[I]).ID;
     result:=max;
end;

function TNode.GetNodeLabel: string;
begin
  result:='ID: '+ inttostr(FID);
end;

function TGraph.IsIn(aList:TList;aNode:TNode):integer;
var
  i: integer;
begin
  result:=-1;
  for i:=0 to aList.Count-1 do
    begin
    if IsEqualData(TNode(aList.Items[i]),aNode) then
      begin
        result:=i;
        exit;
      end;
    end;
end;

  //zatriduje podle Evaluation od nejvetsiho po nejmensi
function TGraph.SortedList(aList:TList;aNode:TNode):integer;
var I:integer;
begin
     result:=-1;
     if aList=nil then exit;
     I:=0;
     while (I<=aList.Count-1)and(aNode.Evaluation < TNode(aList.Items[I]).Evaluation) do inc(I);

     aList.Insert(I,aNode);

     result:=I;
end;



end.

