program Robot;


uses
  Windows,
  Messages,
  SysUtils,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  StdCtrls,
  ExtDlgs,
  Menus,
  ActnMenus,
  SyncObjs,
  OpenGL,
  Math,
  GL,
  Graph in 'Graph.pas',
  GraphRRT in 'GraphRRT.pas',
  Simulator in 'Simulator.pas',
  Polygon in 'Polygon.pas',
  Obstacles in 'Obstacles.pas',
  Distance in 'Distance.pas',
  CollisionDetection in 'CollisionDetection.pas',
  GraphIGPPR in 'GraphIGPPR.pas',
  GraphISSD in 'GraphISSD.pas',
  _RRTdialog in '_RRTdialog.pas' {OKBottomDlg},
  _IGPPRdialog in '_IGPPRdialog.pas' {OKBottomDlg1},
  _ISSDdialog in '_ISSDdialog.pas' {OKBottomDlg2},
  _dlgPrgBar in '_dlgPrgBar.pas' {OKRightDlg},
  _dlgOther in '_dlgOther.pas' {OKBottomDlg3};

type
  TRRT = class (TGraphRRT)
    procedure DrawTreeProc(a,b: TVert);override;
  end;

  TIGPPR = class (TGraphIGPPR)
    procedure DrawTreeProc(a,b: TVert);override;
    procedure DrawCellsProc(a,b: vertex);override;
  end;

  TISSD = class (TGraphISSD)
    procedure DrawTreeProc(a,b: TVert);override;
    procedure DrawCellsProc(a,b: vertex);override;
  end;

var
  OpenDialog: TOpenDialog;
    // MainMenu
  Menu: TMainMenu;
  Item1: TMenuItem;
  Item2: TMenuItem;
  Item11: TMenuItem;
  Item12: TMenuItem;
  Item13: TMenuItem;
  Item14: TMenuItem;
  Item21: TMenuItem;
  Item22: TMenuItem;
  Item23: TMenuItem;
  Item24: TMenuItem;
  Item25: TMenuItem;
  Form1: TForm;
  TreeRRT: TRRT;
  hThread: THandle;
  thid: Cardinal;
  Close: boolean;                  // bezpecne ukonceni
  TimeComp: TDateTime;             // cas vypoctu
  RK: boolean;                     // metoda Runge-Kutta

    // *ISSD* //
  ISSD: TISSD;
  cell_ISSD: boolean;             // kresleni mrizky
  step: integer;                  // pocet spusteni
    // *IGPPR* //
  IGPPR: TIGPPR;
  cell_IGPPR: boolean;            // kresleni mrizky
  MaxDepth: integer;              // max.povolena hloubka pro vyhledavani
  VertexArray: array of vertex;   // pole vrcholu pro kresleni bunek
  CS_cell: TRtlCriticalSection;   // krit.sekce pro pristup k vykresl.bunkam
  CS_obs: TRtlCriticalSection;    // krit.sekce pro pristup k vykresl.prekazkam
    // *RRT* //
  MaxX: integer;                  // velikost plochy (osa x)
  MaxY: integer;                  // velikost plochy (osa y)
  RRT_iter: integer;              // pocet iteraci
  NormalMode: TMode;
  CS_Cars: TRtlCriticalSection;   // Umoznuje threadu docasne blokovat jine thready (synchronizace)
    // Auto
  CarWidth,CarL,CarOver,CarMaxFi,kx,ky,ColX,ColY: double; // parametry robotu
  start,goal: vector_q;           // start.a cil. konfigurace
  EndD: double;                   // cilova tolerance
    // Prekazky
  fObstacles: TObstacles;
    // Simulace
  q: vector_q;        // pozice auta pri jizde
  u: vector_u;        // natoceni auta
  bSimulate: boolean; // vykreslovani auta pri simulaci
  TimeSim: integer;   // prubeh simulace (cas v ms)
    // Vykreslovany strom
  bDrawTree:  boolean;      // vykreslovani stromu
  bPathRRT:   boolean;      // vykreslovani cesty RRT
  bPathIGPPR: boolean;      // vykreslovani cesty IGPPR
  bPathISSD:  boolean;      // vykreslovani cesty ISSD
  TreeArray: TTree;         // pole vertexu
  CS: TRtlCriticalSection;  // kriticka sekce pro pristup k vykreslovanemu stromu

    // OpenGL
  h_Rc: HGLRC;		                  // Trval Rendering Context
  h_Dc: HDC;                        // Privtn GDI Device Context
  h_Wnd: HWND;                      // Obsahuje Handle naeho okna
  keys: array [0..255] of BOOL;	    // Pole pro ukldn vstupu z klvesnice
  Active: bool = true;              // Ponese informaci o tom, zda je okno aktivn
  FullScreen:bool = true;           // Ponese informaci o tom, zda je program ve fullscreenu
  base: GLuint;                     // slo zkladnho display listu znak

{-----------------------------------------------------------------------------}


procedure SetParam;
begin
  RK:=true;
  bDrawTree:=true;
    // RRT
  MaxX:=800;
  MaxY:=600;
  RRT_iter:=10000;
  EndD:=8.0;
  NormalMode.PG:=0.1;
  NormalMode.PG1:=0.5;
  NormalMode.D:=300;
    //ISSD
  MaxDepth:=200;
    // Car
  CarWidth:=15;
  CarL:=15;
  CarOver:=6;
  CarMaxFi:=35*Pi/180;
  kx:=8.0;
  ky:=4;
  ColX:=2;
  ColY:=4;
  TimeSim:=45;
end;

  // funkce threadu provadejici vypocet RRT
function thRRT:dword;
var distance:tdistance;
  i,t,hrany: integer;
  b1,b2,found: boolean;
  cas:double;
begin
  Close:=false;
  OKRightDlg.Caption:='Prbh vpotu';
  OKRightDlg.OKBtn.Caption:='Stop';
  OKRightDlg.Label2.Caption:='0';
  OKRightDlg.Label5.Caption:='0';
  for i:=0 to 10 do begin
    OKRightDlg.ProgressBar1.Position:=(i*10)-3;
    sleep(10);
  end;
  TreeRRT.Stop:=False;
  cell_IGPPR:=false;
  cell_ISSD:=false;
  EnterCriticalSection(cs);
	  Setlength(TreeArray,0);
  LeaveCriticalSection(cs);
  bPathIGPPR:=false;
  bPathISSD:=false;
  bPathRRT:=false;
  q:=vector_q.create(start.x,start.y,start.theta);
  u.fi:=0;
  Item11.Enabled:=false;
  Item12.Enabled:=false;
  Item13.Enabled:=false;
    TimeComp:=GetTime;
  found:=TreeRRT.BuildRRT(start,goal);
  TreeRRT.SetMode(NormalMode);
    // statistika
  TimeComp:= GetTime- TimeComp;
  cas:=TimeComp/0.0000125;  // prevod na sekundy
  cas:=RoundTo(cas, -2);
  OKRightDlg.OKBtn.Caption:='OK';
    // nalezeni cesty
  if not(TreeRRT.Stop) and (found) then
    begin
      TreeRRT.FindPathRRT(goal);
      bPathRRT:=true;
      hrany:=TreeRRT.SolEdges.Count;
      OKRightDlg.Label2.Caption:=FloatToStr(cas);
      OKRightDlg.Label5.Caption:=IntToStr(hrany);
        // Simulace auta
      if bSimulate then
        begin
          OKRightDlg.Close;
          TreeRRT.SolEdges.Add(TreeRRT.SolEdges.Last);
          for i:=0 to TreeRRT.Solution.Count-1 do
            begin
              q:=TreeRRT.Solution.Items[i];
              u:=TreeRRT.SolEdges.Items[i];
              sleep(TimeSim);
            end;
        end;
    end;
  if not(TreeRRT.Stop) and not(found) then OKRightDlg.Caption:=' een nebylo nalezeno...'
      else begin
        sleep(1000);
        OKRightDlg.Close;
      end;
  start:=vector_q.create(0,start.x,start.y,start.theta);
  Item11.Enabled:=true;
  Item12.Enabled:=true;
  Item13.Enabled:=true;
  Close:=true;
  Result:=0;
end;


  // funkce threadu provadejici vypocet IGPPR
function thIGPPR:dword;
var i,hrany: integer;
  found: boolean;
  cas: double;
begin
  Close:=false;
  OKRightDlg.Caption:='Prbh vpotu';
  OKRightDlg.OKBtn.Caption:='Stop';
  OKRightDlg.Label2.Caption:='0';
  OKRightDlg.Label5.Caption:='0';
  for i:=0 to 10 do begin
    OKRightDlg.ProgressBar1.Position:=(i*10)-3;
    sleep(1);
  end;
  IGPPR.Stop:=False;
  cell_ISSD:=false;
  bPathRRT:=false;
  bPathIGPPR:=false;
  bPathISSD:=false;
  Item11.Enabled:=false;
  Item12.Enabled:=false;
  Item13.Enabled:=false;
  EnterCriticalSection(CS);
    Setlength(TreeArray,0);
  LeaveCriticalSection(CS);
  cell_IGPPR:=IGPPR.bDrawCells;
  TimeComp:=GetTime;
  EnterCriticalSection(CS_cell);
    Setlength(VertexArray,0);
    IGPPR.CreateCostMap;
  LeaveCriticalSection(CS_cell);
  found:=IGPPR.IGPPR(start,goal,MaxDepth);
    // statistika
  TimeComp:= GetTime- TimeComp;
  cas:=TimeComp/0.0000125;  // prevod na sekundy
  cas:=RoundTo(cas, -2);
  OKRightDlg.OKBtn.Caption:='OK';
  if not(IGPPR.Stop) and (found) then
    begin
      IGPPR.FindPath;
      bPathIGPPR:=true;
      hrany:=IGPPR.SolEdges.Count;
      OKRightDlg.Label2.Caption:=FloatToStr(cas);
      OKRightDlg.Label5.Caption:=IntToStr(hrany);
          // Simulace auta
      if bSimulate then
      begin
        OKRightDlg.Close;
        IGPPR.SolEdges.Add(IGPPR.SolEdges.Last);
        for i:=0 to IGPPR.Solution.Count-1 do
          begin
            q:=IGPPR.Solution.Items[i];
            u:=IGPPR.SolEdges.Items[i];
            sleep(TimeSim);
          end;
      end;
    end else IGPPR.ResetSol;
  if not(IGPPR.Stop) and not(found) then OKRightDlg.Caption:=' een nebylo nalezeno...'
      else begin
        sleep(1000);
        OKRightDlg.Close;
      end;
  start:=vector_q.create(0,start.x,start.y,0);
  Item11.Enabled:=true;
  Item12.Enabled:=true;
  Item13.Enabled:=true;
  Close:=true;
  Result:=0;
end;


  // funkce threadu provadejici vypocet ISSD
function thISSD:dword;
var i,hrany: integer;
  found: boolean;
  cas: double;
begin
  Close:=false;
  OKRightDlg.Caption:='Prbh vpotu';
  OKRightDlg.OKBtn.Caption:='Stop';
  OKRightDlg.Label2.Caption:='0';
  OKRightDlg.Label5.Caption:='0';
  for i:=0 to 10 do begin
    OKRightDlg.ProgressBar1.Position:=(i*10)-3;
    sleep(0);
  end;
  ISSD.Stop:=False;
  cell_IGPPR:=false;
  bPathRRT:=false;
  bPathIGPPR:=false;
  bPathISSD:=false;
  Item11.Enabled:=false;
  Item12.Enabled:=false;
  Item13.Enabled:=false;
  step:=step+1;
  EnterCriticalSection(CS);
	  Setlength(TreeArray,0);
  LeaveCriticalSection(CS);
  cell_ISSD:=ISSD.bDrawCells;
  TimeComp:=GetTime;
  EnterCriticalSection(CS_cell);
    Setlength(VertexArray,0);
    if (step=1) then ISSD.Decomposing
      else ISSD.Default;
  LeaveCriticalSection(CS_cell);
  found:=ISSD.ISSD(start,goal,MaxDepth);
      // statistika
  TimeComp:= GetTime- TimeComp;
  cas:=TimeComp/0.0000125;  // prevod na sekundy
  cas:=RoundTo(cas, -2);
  OKRightDlg.OKBtn.Caption:='OK';
  if not(ISSD.Stop) and (found) then
    begin
      ISSD.FindPath;
      bPathISSD:=true;
      hrany:=ISSD.SolEdges.Count;
      OKRightDlg.Label2.Caption:=FloatToStr(cas);
      OKRightDlg.Label5.Caption:=IntToStr(hrany);
          // Simulace auta
      if bSimulate then
      begin
        OKRightDlg.Close;
        ISSD.SolEdges.Add(ISSD.SolEdges.Last);
        for i:=0 to ISSD.Solution.Count-1 do
          begin
            q:=ISSD.Solution.Items[i];
            u:=ISSD.SolEdges.Items[i];
            sleep(TimeSim);
          end;
      end;
    end else ISSD.ResetSol;

    if not(ISSD.Stop) and not(found) then OKRightDlg.Caption:=' een nebylo nalezeno...'
      else begin
        sleep(1000);
        OKRightDlg.Close;
      end;

  start:=vector_q.create(0,start.x,start.y,0);
  Item11.Enabled:=true;
  Item12.Enabled:=true;
  Item13.Enabled:=true;
  Close:=true;
  Result:=0;
end;

//------------------------------------------------------------------------------------------
// Vypis textu
//------------------------------------------------------------------------------------------
procedure glPrint(text: string);
begin
  if text = '' then exit;                                 // Byl pedn text?
  glPushAttrib(GL_LIST_BIT);                              // Ulo souasn stav display list
  glListBase(base - 32);                                  // Nastav zkladn znak na 32
  glCallLists(length(text),GL_UNSIGNED_BYTE,Pchar(text)); // Vykresl display listy
  glPopAttrib;                                            // Obnov pvodn stav display list
end;

procedure DrawEvalCells();
var i,j,w,l: integer;
  str: string;
  x,y: integer;
begin
	glLoadIdentity();
	glColor3f(0.0, 0.0, 1.0);

  EnterCriticalSection(CS_cell);
  w:=trunc(IGPPR.MaxX/IGPPR.dt);
  l:=trunc(IGPPR.MaxY/IGPPR.dt);
  for i:=0 to l-1 do
    for j:=0 to w-1 do
      begin
        str:=inttostr(TCell(IGPPR.D[i,j]).evaluation);
        x:=j*IGPPR.dt+trunc(IGPPR.dt/5);
        y:=i*IGPPR.dt+trunc(IGPPR.dt/2);
        glRasterPos2f(x,y); // Pozice textu
        glPrint(str);       // Vpis textu
      end;
  LeaveCriticalSection(CS_cell);
end;

procedure DrawOccCells();
var i,j,w,l: integer;
  str: string;
  x,y: integer;
begin
	glLoadIdentity();
	glColor3f(0.0, 0.0, 1.0);

  EnterCriticalSection(CS_cell);
  w:=trunc(ISSD.MaxX/ISSD.dt);
  l:=trunc(ISSD.MaxY/ISSD.dt);
  for i:=0 to l-1 do
    for j:=0 to w-1 do
      begin
        str:=booltostr(TCell2(ISSD.D[i,j]).occupied);
        x:=j*ISSD.dt+trunc(ISSD.dt/5);
        y:=i*ISSD.dt+trunc(ISSD.dt/2);
        glRasterPos2f(x,y); // Pozice textu
        glPrint(str);       // Vpis textu
      end;
  LeaveCriticalSection(CS_cell);
end;


//------------------------------------------------------------------------------------------
//                            {O P E N  G L}
//------------------------------------------------------------------------------------------

  // Vytvoen fontu
procedure BuildFont;
var Font: HFONT;                                          // Promnn fontu
begin
  base := glGenLists(96);                                 // 96 znak
  font := CreateFont(-9,                                 // Vka
                      0,                                  // ka
                      0,                                  // hel escapement
                      0,                                  // hel orientace
                      FW_BOLD,                            // Tunost
                      0,                                  // Kurzva
                      0,                                  // Podtren
                      0,                                  // Pekrtnut
                      ANSI_CHARSET,                       // Znakov sada
                      OUT_TT_PRECIS,                      // Pesnost vstupu (TrueType)
                      CLIP_DEFAULT_PRECIS,                // Pesnost oezn
                      ANTIALIASED_QUALITY,                // Vstupn kvalita
                      FF_DONTCARE or DEFAULT_PITCH,       // Rodina a pitch
                      'Courier New');                     // Jmno fontu
  SelectObject(h_dc,font);                                // Vbr fontu do DC
  wglUseFontBitmaps(h_dc,32,96,base);                     // Vytvo 96 znak, ponaje 32 v Ascii
end;

//------------------------------------------------------------------------------------------
// Vykresleni stromu
//------------------------------------------------------------------------------------------
procedure DrawTree();
var i,l:integer;
begin
  i:=sizeof(TVert);
	glLoadIdentity();
	glColor3f(1.0, 0.0, 0.0);
  EnterCriticalSection(cs);
 try
  l:=length(TreeArray);
  glVertexPointer(2, GL_INT, i, TreeArray);     // ukazatel na pole vrcholu
  glDrawArrays(GL_LINES, 0, l-1);               // kresleni stromu
 except
  raise Exception.Create('Vertex Array Error');
  exit;
 end;
  LeaveCriticalSection(cs);
end;

//------------------------------------------------------------------------------------------
// Pridava nove hrany do vykreslovaneho stromu
//------------------------------------------------------------------------------------------
procedure TRRT.DrawTreeProc(a,b: TVert);
var l: integer;
begin
  EnterCriticalSection(cs);
    l:=length(TreeArray);
    setlength(TreeArray,l+2);
    TreeArray[l]:=a;
    TreeArray[l+1]:=b;
  LeaveCriticalSection(cs);
end;

procedure TIGPPR.DrawTreeProc(a,b: TVert);
var l: integer;
begin
  EnterCriticalSection(cs);
    l:=length(TreeArray);
    setlength(TreeArray,l+2);
    TreeArray[l]:=a;
    TreeArray[l+1]:=b;
  LeaveCriticalSection(cs);
end;

procedure TISSD.DrawTreeProc(a,b: TVert);
var l: integer;
begin
  EnterCriticalSection(cs);
    l:=length(TreeArray);
    setlength(TreeArray,l+2);
    TreeArray[l]:=a;
    TreeArray[l+1]:=b;
  LeaveCriticalSection(cs);
end;

//------------------------------------------------------------------------------------------
// Vykresleni mrizky
//------------------------------------------------------------------------------------------
procedure DrawCells();
var i,l:integer;
begin
  i:=sizeof(vertex);
	glLoadIdentity();
	glColor3f(0.9, 0.9, 0.9);
  EnterCriticalSection(CS_cell);
 try
  l:=length(VertexArray);
  glVertexPointer(2, GL_DOUBLE, i, VertexArray);  // ukazatel na pole vrcholu
  glDrawArrays(GL_LINES, 0, l-1);                 // kresleni stromu
 except
  raise Exception.Create('Array of Cells Error');
  exit;
 end;
  LeaveCriticalSection(CS_cell);
end;

//------------------------------------------------------------------------------------------
// Pridava nove hrany do mrzky
//------------------------------------------------------------------------------------------
procedure TIGPPR.DrawCellsProc(a,b: vertex);
var l: integer;
begin
  //EnterCriticalSection(CS_cell);
    l:=length(VertexArray);
    setlength(VertexArray,l+2);
    VertexArray[l]:=a;
    VertexArray[l+1]:=b;
  //LeaveCriticalSection(CS_cell);
end;

procedure TISSD.DrawCellsProc(a,b: vertex);
var l: integer;
begin
  //EnterCriticalSection(CS_cell);
    l:=length(VertexArray);
    setlength(VertexArray,l+2);
    VertexArray[l]:=a;
    VertexArray[l+1]:=b;
  //LeaveCriticalSection(CS_cell);
end;

//------------------------------------------------------------------------------------------
// Vykresleni cesty
//------------------------------------------------------------------------------------------
procedure DrawPath(ArrayofVertex:array of TVert);
var l,i: integer;
begin
	glLoadIdentity();
  i:=sizeof(TVert);
	glColor3f(0.0, 0.0, 1.0);
	glLineWidth(3.0);
  l:=Length(ArrayofVertex);
	glVertexPointer(2, GL_INT, i, @ArrayofVertex);
	glDrawArrays(GL_LINES, 0, l-1);
	glLineWidth(1.0);
end;

//------------------------------------------------------------------------------------------
// Vykresleni prekazek
//------------------------------------------------------------------------------------------
procedure DrawObstacles(ObsList:TList);
var i,j: integer;
    x,y: double;
begin
	glLoadIdentity();
	glColor3f(0.0, 0.0, 0.0);
  glLineWidth(1.5);
 EnterCriticalSection(CS_obs);
  for i:=0 to ObsList.Count-1 do
    begin
		  glBegin(GL_LINE_LOOP);
        for j:=0 to TPolygon(ObsList.Items[i]).VertexCount-1 do
          begin
            x:=TVertex(TPolygon(ObsList.Items[i]).Vertices.Items[j]).x;
            y:=TVertex(TPolygon(ObsList.Items[i]).Vertices.Items[j]).y;
            glVertex2d(x,y);
          end;
	  	glEnd();
    end;
  glLineWidth(1.0);
 LeaveCriticalSection(CS_obs);
end;

//------------------------------------------------------------------------------------------
// Vykresleni pocatecni pozice auta
//------------------------------------------------------------------------------------------
procedure DrawStartCar;
begin
  glLoadIdentity();
	EnterCriticalSection(cs_CARS);

	glTranslated(start.x, start.y, 0.0);
	glRotated(start.theta * 180 / PI, 0.0, 0.0, 1.0);

	LeaveCriticalSection(cs_CARS);

	glColor3f(0.0, 0.0, 0.0);
	glBegin(GL_POINTS);
		glVertex2d(0.0, 0.0);
	glEnd();

	glLineWidth(2.0);
	glColor3f(0.0, 1.0, 0.0);
	glBegin(GL_LINE_LOOP);
		glVertex2d(-(CarOver + ColX), (CarWidth / 2) + ColY);
		glVertex2d(CarL + CarOver + ColX,  (CarWidth / 2) + ColY);
		glVertex2d(CarL + CarOver + ColX,  -(CarWidth / 2) - ColY);
		glVertex2d(-(CarOver + ColX), -(CarWidth / 2) - ColY);
	glEnd();
	glLineWidth(1.0);
end;

//------------------------------------------------------------------------------------------
// Vykresleni cilove pozice auta
//------------------------------------------------------------------------------------------
procedure DrawGoalCar;
begin
  glLoadIdentity();
	EnterCriticalSection(cs_CARS);

	glTranslated(goal.x, goal.y, 0.0);
	glRotated(goal.theta * 180 / PI, 0.0, 0.0, 1.0);

	LeaveCriticalSection(cs_CARS);

	glColor3f(0.0, 0.0, 0.0);
  glPointSize(5.0);
  glEnable(GL_POINT_SMOOTH);
	glBegin(GL_POINTS);
		glVertex2d(0.0, 0.0);
	glEnd();

	glLineWidth(2.0);
	glColor3f(1.0, 0.0, 0.0);
	glBegin(GL_LINE_LOOP);
		glVertex2d(-(CarOver + ColX), (CarWidth / 2) + ColY);
		glVertex2d(CarL + CarOver + ColX,  (CarWidth / 2) + ColY);
		glVertex2d(CarL + CarOver + ColX,  -(CarWidth / 2) - ColY);
		glVertex2d(-(CarOver + ColX), -(CarWidth / 2) - ColY);
	glEnd();
	glLineWidth(1.0);
end;

//------------------------------------------------------------------------------------------
// Vykresleni kol auta pri simulaci
//------------------------------------------------------------------------------------------
procedure DrawWheels;
begin
	// Vykresleni praveho predniho kola
	glLoadIdentity();
	glTranslated(q.x, q.y, 0.0);
	glRotated(q.theta*180/PI, 0.0, 0.0, 1.0);

	glTranslated(CarL, CarWidth / 2, 0.0);
	glRotated(u.fi*180.0/PI, 0.0, 0.0, 1.0);

	glColor3f(0.2, 0.2, 0.2);
	glBegin(GL_QUADS);
		glVertex2d(kx/2,ky/2);
		glVertex2d(kx/2,-ky/2);
		glVertex2d(-kx/2,-ky/2);
		glVertex2d(-kx/2,ky/2);
	glEnd();

	// Vykresleni leveho predniho kola
	glLoadIdentity();
	glTranslated(q.x, q.y, 0.0);
	glRotated(q.theta*180/PI, 0.0, 0.0, 1.0);

	glTranslated(CarL, -CarWidth / 2, 0.0);
	glRotated(u.fi*180.0/PI, 0.0, 0.0, 1.0);

	glBegin(GL_QUADS);								
		glVertex2d(kx/2,ky/2);					
		glVertex2d(kx/2,-ky/2);
		glVertex2d(-kx/2,-ky/2);
		glVertex2d(-kx/2,ky/2);
	glEnd();

	// Vykresleni leveho zadniho kola
	glLoadIdentity();
	glTranslated(q.x, q.y, 0.0);
	glRotated(q.theta*180/PI, 0.0, 0.0, 1.0);

	glTranslated(0.0, -CarWidth / 2, 0.0);

	glBegin(GL_QUADS);
		glVertex2d(kx/2,ky/2);
		glVertex2d(kx/2,-ky/2);
		glVertex2d(-kx/2,-ky/2);
		glVertex2d(-kx/2,ky/2);
	glEnd();

	// Vykresleni praveho zadniho kola
	glLoadIdentity();
	glTranslated(q.x, q.y, 0.0);
	glRotated(q.theta*180/PI, 0.0, 0.0, 1.0);

	glTranslated(0.0, CarWidth / 2, 0.0);

	glBegin(GL_QUADS);
		glVertex2d(kx/2,ky/2);
		glVertex2d(kx/2,-ky/2);
		glVertex2d(-kx/2,-ky/2);
		glVertex2d(-kx/2,ky/2);
	glEnd();
end;

//------------------------------------------------------------------------------------------
// Vykresleni auta pri simulaci
//------------------------------------------------------------------------------------------
procedure DrawCar();
begin
	glLoadIdentity();
	glTranslated(q.x, q.y, 0.0);
	glRotated(q.theta * 180 / PI, 0.0, 0.0, 1.0);

	glColor3f(0.0, 0.7, 0.0);
	glBegin(GL_POLYGON);
		glVertex2d(-CarOver, CarWidth / 2);
		glVertex2d(CarL + CarOver,  CarWidth / 2);
		glVertex2d(CarL + CarOver,  -CarWidth / 2);
		glVertex2d(-CarOver, -CarWidth / 2);	
	glEnd();

	glColor3f(1.0, 1.0, 1.0);
	glBegin(GL_POINTS);
		glVertex2d(0.0, 0.0);
	glEnd();
end;



//------------------------------------------------------------------------------------------
// OpenGL
//------------------------------------------------------------------------------------------

procedure ReSizeGLScene(Width: GLsizei; Height: GLsizei); // Zmna velikosti a inicializace OpenGL okna
begin
  if (Height=0) then		                                  // Zabezpeen proti dlen nulou
     Height:=1;                                           // Nastav vku na jedna
  glViewport(0, 0, Width, Height);                        // Resetuje aktuln nastaven
  glMatrixMode(GL_PROJECTION);                            // Zvol projekn matici
  glLoadIdentity();                                       // Reset matice
  glOrtho(0.0,width,height,0.0,-1.0,1.0);                 // Spocita matici pro kolmou projekci
  //gluPerspective(45.0,Width/Height,0.1,100.0);          // Vpoet perspektivy
  glMatrixMode(GL_MODELVIEW);                             // Zvol matici Modelview
  glLoadIdentity;                                         // Reset matice
end;


function InitGL:bool;	                                // Vechno nastaven OpenGL
begin
  glShadeModel(GL_SMOOTH);			                      // Povol jemn stnovn
  glClearColor(1.0, 1.0, 1.0, 1.0);
  //glClearColor(0.0, 0.4, 0.6, 0.5);	  	            // Pozad
  glClearDepth(1.0);				                          // Nastaven hloubkovho bufferu
  //glEnable(GL_DEPTH_TEST);			                    // Povol hloubkov testovn
  //glDepthFunc(GL_LEQUAL);				                    // Typ hloubkovho testovn
  //glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST); // Nejlep perspektivn korekce
  glEnableClientState(GL_VERTEX_ARRAY);               // Aktivuje pole bod
  BuildFont;
  Result:=true;                                       // Inicializace probhla v podku
end;

function DrawGLScene():bool;                            // Vykreslovn
begin
  glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);  // Smae obrazovku a hloubkov buffer
  glLoadIdentity();
  if cell_IGPPR then begin
      DrawCells;
      DrawEvalCells;
    end;
  if cell_ISSD then begin
      DrawCells;
      DrawOccCells;
    end;
  DrawObstacles(fObstacles.Obstacles);
  DrawTree;
  DrawStartCar;
  DrawGoalCar;
  if bPathRRT   then DrawPath(TreeRRT.SolTree);
  if bPathIGPPR then DrawPath(IGPPR.SolTree);
  if bPathISSD  then DrawPath(ISSD.SolTree);
  glColor3f(1.0, 0.0, 0.0);
  if bSimulate then
    begin
      DrawCar;
      DrawWheels;
    end;
  Result := true;                                       // Vykreslen probhlo v podku
end;

  // inicializace dialog.oken
procedure InitRRT;
begin
  OKBottomDlg.Edit1.Text:=FloatToStr(TreeRRT.EndD);
  OKBottomDlg.Edit2.Text:=FloatToStr(TreeRRT.NormalMode.PG);
  OKBottomDlg.Edit3.Text:=FloatToStr(TreeRRT.NormalMode.PG1);
  OKBottomDlg.Edit4.Text:=FloatToStr(TreeRRT.NormalMode.D);
  OKBottomDlg.Edit5.Text:=FloatToStr(TreeRRT.GoalMode.PG);
  OKBottomDlg.Edit6.Text:=FloatToStr(TreeRRT.GoalMode.PG1);
  OKBottomDlg.Edit7.Text:=FloatToStr(TreeRRT.GoalMode.D);
  OKBottomDlg.Edit8.Text:=IntToStr(TreeRRT.Rep);
  OKBottomDlg.Edit9.Text:=IntToStr(TreeRRT.dt);
  OKBottomDlg.Edit10.Text:=IntToStr(TreeRRT.Iter);
end;

procedure InitIGPPR;
begin
  OKBottomDlg1.Edit1.Text:=FloatToStr(IGPPR.EndD);
  OKBottomDlg1.Edit2.Text:=IntToStr(MaxDepth);
  OKBottomDlg1.Edit3.Text:=IntToStr(IGPPR.dt);
  OKBottomDlg1.Edit4.Text:=IntToStr(IGPPR.dti);
  if (IGPPR.CostMap=4) then OKBottomDlg1.RadioButton1.Checked:=true
    else OKBottomDlg1.RadioButton2.Checked:=true;
end;

procedure InitISSD;
begin
  OKBottomDlg2.Edit1.Text:=FloatToStr(ISSD.EndD);
  OKBottomDlg2.Edit2.Text:=IntToStr(MaxDepth);
  OKBottomDlg2.Edit3.Text:=IntToStr(ISSD.dt);
  OKBottomDlg2.Edit4.Text:=IntToStr(ISSD.dti);
end;

procedure InitOther;
begin
  if RK then OKBottomDlg3.RadioButton2.Checked:=true
    else OKBottomDlg3.RadioButton1.Checked:=true;
  if bDrawTree then OKBottomDlg3.CheckBox1.Checked:=true
    else OKBottomDlg3.CheckBox1.Checked:=false;
  OKBottomDlg3.Edit2.Text:=IntToStr(TimeSim);
end;

  // nastaveni parametru dialog.oken
procedure SetRRT;
var DrawTree,Eul: boolean;
   EndD,PG,PG1,D: double;
   Rep,Krok,Iter: integer;
   Mode: TMode;
begin
  EndD:=OKBottomDlg.StrToVal(OKBottomDlg.Edit1);
    if EndD>4 then TreeRRT.EndD:=EndD else
      TreeRRT.EndD:=6.0;
  Rep:=trunc(OKBottomDlg.StrToVal(OKBottomDlg.Edit8));
    if not(Rep=-1) then TreeRRT.Rep:=Rep;
  Krok:=trunc(OKBottomDlg.StrToVal(OKBottomDlg.Edit9));
    if not(Krok=-1) and (Krok>2) and (Krok<16) then TreeRRT.dt:=Krok;
  Iter:=trunc(OKBottomDlg.StrToVal(OKBottomDlg.Edit10));
    if not(Iter=-1) and (Iter>0) and (Iter<30000) then TreeRRT.Iter:=Iter;
  PG:=OKBottomDlg.StrToVal(OKBottomDlg.Edit2);
  PG1:=OKBottomDlg.StrToVal(OKBottomDlg.Edit3);
  D:=OKBottomDlg.StrToVal(OKBottomDlg.Edit4);
  if not(PG=-1)and not(PG=-1)and not(D=-1)and (PG<1.1)and (PG1<1.1) then
  begin
    Mode.PG:=PG; Mode.PG1:=PG1;
    NormalMode.PG:=PG; NormalMode.PG1:=PG1;
      if (D>29) then begin
        Mode.D:=D;
        NormalMode.D:=D;
       end else begin
        Mode.D:=50;
        NormalMode.D:=50;
        end;
    TreeRRT.SetMode(Mode);
  end;
  PG:=OKBottomDlg.StrToVal(OKBottomDlg.Edit5);
  PG1:=OKBottomDlg.StrToVal(OKBottomDlg.Edit6);
  D:=OKBottomDlg.StrToVal(OKBottomDlg.Edit7);
  if not(PG=-1)and not(PG=-1)and not(D=-1)and (PG<1.1)and (PG1<1.1) then
  begin
    Mode.PG:=PG; Mode.PG1:=PG1;
      if (D>29) then Mode.D:=D else
        Mode.D:=50;
    TreeRRT.SetGoalMode(Mode);
  end;
end;

procedure SetIGPPR;
var Depth,Cell,dti: integer;
    EndD: double;
    Method: boolean;
begin
  Method:=OKBottomDlg1.RadioButton1.Checked;
    if Method then IGPPR.CostMap:=4
      else IGPPR.CostMap:=8;
  EndD:=OKBottomDlg.StrToVal(OKBottomDlg1.Edit1);
    if EndD>4 then IGPPR.EndD:=EndD else
      IGPPR.EndD:=6.0;
  Depth:=trunc(OKBottomDlg.StrToVal(OKBottomDlg1.Edit2));
    if not(Depth=-1) then MaxDepth:=Depth;
  Cell:=trunc(OKBottomDlg.StrToVal(OKBottomDlg1.Edit3));
    if not(Cell=-1) and (Cell>4) and (Cell<26) then IGPPR.dt:=Cell;
  dti:=trunc(OKBottomDlg.StrToVal(OKBottomDlg1.Edit4));
    if not(dti=-1) and (dti>4) and (dti<31) then IGPPR.dti:=dti;
end;

procedure SetISSD;
var EndD: double;
    Cell,dti,Depth: integer;
begin
  EndD:=OKBottomDlg.StrToVal(OKBottomDlg2.Edit1);
    if EndD>4 then ISSD.EndD:=EndD else
      ISSD.EndD:=6.0;
  Depth:=trunc(OKBottomDlg.StrToVal(OKBottomDlg2.Edit2));
    if not(Depth=-1) then MaxDepth:=Depth;
  Cell:=trunc(OKBottomDlg.StrToVal(OKBottomDlg2.Edit3));
    if not(Cell=-1) and (Cell>4) and (Cell<26) then ISSD.dt:=Cell;
  dti:=trunc(OKBottomDlg.StrToVal(OKBottomDlg2.Edit4));
    if not(dti=-1) and (dti>4) and (dti<31) then ISSD.dti:=dti;
end;

procedure SetOther;
var DrawTree,DrawCell,Eul: boolean;
    Tsim:integer;
begin
  DrawTree:=OKBottomDlg3.CheckBox1.Checked;
    TreeRRT.bDrawTree:=DrawTree;
    IGPPR.bDrawTree:=DrawTree;
    ISSD.bDrawTree:=DrawTree;
    bDrawTree:=DrawTree;
  DrawCell:=OKBottomDlg3.CheckBox2.Checked;
    IGPPR.bDrawCells:=DrawCell;
    ISSD.bDrawCells:=DrawCell;
    if not(DrawCell) then begin
      cell_IGPPR:=false;
      cell_ISSD:=false;
    end;
  CarMaxFi:=OKBottomDlg.StrToVal(OKBottomDlg3.Edit1);
  if not(CarMaxFi=-1) and (CarMaxFi<91) then
  begin
    CarMaxFi:=CarMaxFi*(Pi/180);
    TreeRRT.MaxFi:=CarMaxFi;
    IGPPR.MaxFi:=CarMaxFi;
    ISSD.MaxFi:=CarMaxFi;
  end;
  Eul:=OKBottomDlg3.RadioButton1.Checked;
    TreeRRT.xSimulator.Method:=Eul;
    IGPPR.xSimulator.Method:=Eul;
    ISSD.xSimulator.Method:=Eul;
    RK:=not(Eul);
  Tsim:=trunc(OKBottomDlg.StrToVal(OKBottomDlg3.Edit2));
    if not(Tsim=-1) and (Tsim<1000) then TimeSim:=Tsim;
  bSimulate:=OKBottomDlg3.CheckBox3.Checked;

end;

function WndProc(hWnd: HWND;                            // Handle okna
                 message: UINT;                         // Zprva pro okno
                 wParam: WPARAM;                        // Doplkov informace
                 lParam: LPARAM):                       // Doplkov informace
                                  LRESULT; stdcall;
var x,i: integer;
begin
  if message=WM_SYSCOMMAND then                         // Systmov pkaz
    begin
      case wParam of                                    // Typ systmovho pkazu
        SC_SCREENSAVE,SC_MONITORPOWER:                  // Pokus o zapnut etie obrazovky, Pokus o pechod do spornho reimu?
          begin
            result:=0;                                  // Zabrn obojmu
            exit;
          end;
      end;
    end;
  case message of                                       // Vtven podle pchoz zprvy
    WM_ACTIVATE:                                        // Zmna aktivity okna
      begin
        if (Hiword(wParam)=0) then                      // Zkontroluje zda nen minimalizovan
          active:=true                                  // Program je aktivn
        else
          active:=false;                                // Program nen aktivn
        Result:=0;                                      // Nvrat do hlavnho cyklu programu
      end;
    WM_CLOSE:                                           // Povel k ukonen programu
      Begin
        if (Close=True) then
        begin
          PostQuitMessage(0);                             // Pole zprvu o ukonen
          fObstacles.Destroy;
          TreeRRT.Destroy;
          IGPPR.Destroy;
          ISSD.Destroy;
        end;
        result:=0                                       // Nvrat do hlavnho cyklu programu
      end;
    WM_KEYDOWN:                                         // Stisk klvesy
      begin
        keys[wParam] := TRUE;                           // Oznm to programu
        result:=0;                                      // Nvrat do hlavnho cyklu programu
      end;
    WM_MOUSEWHEEL:
      begin
        if not(LOWORD(wParam)=MK_CONTROL) then
          begin // Pocatecni natoceni robota
            if (wParam>0) then start.theta:= start.theta + 0.1
              else start.theta:= start.theta - 0.1;
				    if (abs(start.theta) >= 2 * PI) then
					    start.theta := 0.0;
          end
			  else
          begin // Cilove natoceni robota
            if (wParam>0) then goal.theta:= goal.theta + 0.1
              else goal.theta:= goal.theta - 0.1;
					  if (abs(goal.theta) >= 2 * PI) then
						goal.theta := 0.0;
          end;
			result:=0;
      end;
    WM_LBUTTONDOWN:
      begin
			  start.x := LOWORD(lParam);
			  start.y := HIWORD(lParam);
        q:=start; u.fi:=0;
        result:=0;
      end;
    WM_RBUTTONDOWN:
      begin
			  goal.x := LOWORD(lParam);
			  goal.y := HIWORD(lParam);
        result:=0;
      end;
    WM_KEYUP:                                           // Uvolnn klvesy
      begin
    	keys[wParam] := FALSE;                            // Oznm to programu
        result:=0;                                      // Nvrat do hlavnho cyklu programu
      end;
    WM_PAINT:
      begin
        DrawGLScene();
        SwapBuffers(h_DC);
      end;
    WM_COMMAND:       // Vbr z Menu
      begin
        if (wParam=3) then                              // 1.polozka RRT
          begin
              // vytvoreni threadu
            hThread:=BeginThread(nil,0,@thRRT,nil,0,thid);
            if (hThread=0) then
              begin
                MessageBox(h_Wnd, 'CreateThread RRT function error.', 'Error', MB_ICONEXCLAMATION or MB_OK);
                result:=0;
              end;
            if (OKRightDlg.ShowModal)=mrOK then TreeRRT.Stop:=True;
            result:=0;
          end;

        if (wParam=4) then                              // 2.polozka IGPPR
          begin
              // vytvoreni threadu
            hThread:=BeginThread(nil,0,@thIGPPR,nil,0,thid);
            if (hThread=0) then
              begin
                MessageBox(h_Wnd, 'CreateThread IGPPR function error.', 'Error', MB_ICONEXCLAMATION or MB_OK);
                result:=0;
              end;
            if (OKRightDlg.ShowModal)=mrOK then IGPPR.Stop:=True;
            result:=0;
          end;

        if (wParam=5) then                              // 3.polozka ISSD
          begin
              // vytvoreni threadu
            hThread:=BeginThread(nil,0,@thISSD,nil,0,thid);
            if (hThread=0) then
              begin
                MessageBox(h_Wnd, 'CreateThread ISSD function error.', 'Error', MB_ICONEXCLAMATION or MB_OK);
                result:=0;
              end;
            if (OKRightDlg.ShowModal)=mrOK then ISSD.Stop:=True;
            result:=0;
          end;

        if (wParam=6) then                              // 4.polozka Close
          begin
            SendMessage(h_Wnd,WM_CLOSE, 0, 0);
          end;
                                                        // 5.polozka Set RRT
        if (wParam=7) then
          begin
            OKBottomDlg:=TOKBottomDlg.Create(nil);
            try
              InitRRT;
              if (OKBottomDlg.ShowModal)=mrOK then SetRRT;
            finally
              OKBottomDlg.Free;
            end;
          end;

        if (wParam=8) then
          begin                                         // 6.polozka Set IGPPR
            OKBottomDlg1:=TOKBottomDlg1.Create(nil);
            try
              InitIGPPR;
              if (OKBottomDlg1.ShowModal)=mrOK then
              begin
                SetIGPPR;
                cell_IGPPR:=false;
                cell_ISSD:=false;
              end;
            finally
              OKBottomDlg1.Free;
            end;
          end;

        if (wParam=9) then                              // 7.polozka Set ISSD
          begin
            OKBottomDlg2:=TOKBottomDlg2.Create(nil);
            try
              InitISSD;
              if (OKBottomDlg2.ShowModal)=mrOK then
                begin
                  SetISSD;
                  step:=0;
                  cell_IGPPR:=false;
                  cell_ISSD:=false;
                end;
            finally
              OKBottomDlg2.Free;
            end;
          end;

        if (wParam=10) then                              // 8.polozka Ostatni
          begin
            OKBottomDlg3:=TOKBottomDlg3.Create(nil);
            try
              InitOther;
              if (OKBottomDlg3.ShowModal)=mrOK then SetOther;
              if (OKBottomDlg3.CheckBox3.Checked) then OKBottomDlg3.Edit2.Enabled:=True;
            finally
              OKBottomDlg3.Free;
            end;
          end;

        if (wParam=11) then                              // 9.polozka Load Obstacles
          begin
            OpenDialog:=TOpenDialog.Create(nil);
            OpenDialog.Filter:='Obstacles (*.obs)|*.obs|All Files (*.*)|*.*';
            if (OpenDialog.Execute) then
              begin
                EnterCriticalSection(CS_obs);
                  if fObstacles.Load(OpenDialog.FileName)=1 then
                    begin
                      step:=0;
                      cell_IGPPR:=false;
                      cell_ISSD:=false;
                      bPathIGPPR:=false;
                      bPathISSD:=false;
                      bPathRRT:=false;
                      EnterCriticalSection(cs);
	                      Setlength(TreeArray,0);
                      LeaveCriticalSection(cs);
                    end;
                LeaveCriticalSection(CS_obs);
              end;
          end;
      end;
    WM_SIZe:                                            // Zmna velikosti okna
      begin
    	ReSizeGLScene(LOWORD(lParam),HIWORD(lParam));     // LoWord=ka, HiWord=Vka
        result:=0;                                      // Nvrat do hlavnho cyklu programu
      end
    else
      // Pedn ostatnch zprv systmu
      begin
      	Result := DefWindowProc(hWnd, message, wParam, lParam);
      end;
    end;
end;


procedure KillGLWindow;                                 // Zavrn okna
begin
  if FullScreen then                                    // Jsme ve fullscreenu?
    begin
      ChangeDisplaySettings(devmode(nil^),0);           // Pepnut do systmu
      showcursor(true);                                 // Zobraz kurzor myi
    end;
  if h_rc<> 0 then                                      // Mme rendering kontext?
    begin
      if (not wglMakeCurrent(h_Dc,0)) then              // Jsme schopni oddlit kontexty?
        MessageBox(0,'Release of DC and RC failed.',' Shutdown Error',MB_OK or MB_ICONERROR);
      if (not wglDeleteContext(h_Rc)) then              // Jsme schopni smazat RC?
        begin
          MessageBox(0,'Release of Rendering Context failed.',' Shutdown Error',MB_OK or MB_ICONERROR);
          h_Rc:=0;                                      // Nastav hRC na 0
        end;
    end;
  if (h_Dc=1) and (releaseDC(h_Wnd,h_Dc)<>0) then       // Jsme schopni uvolnit DC
    begin
      MessageBox(0,'Release of Device Context failed.',' Shutdown Error',MB_OK or MB_ICONERROR);
      h_Dc:=0;                                          // Nastav hDC na 0
    end;
  if (h_Wnd<>0) and (not destroywindow(h_Wnd))then      // Jsme schopni odstranit okno?
    begin
      MessageBox(0,'Could not release hWnd.',' Shutdown Error',MB_OK or MB_ICONERROR);
      h_Wnd:=0;                                         // Nastav hWnd na 0
    end;
  if (not windows.UnregisterClass('OpenGL',hInstance)) then     // Jsme schopni odregistrovat tdu okna?
    begin
      MessageBox(0,'Could Not Unregister Class.','SHUTDOWN ERROR',MB_OK or MB_ICONINFORMATION);
    end;
end;


function CreateGlWindow(title:Pchar; width,height,bits:integer;FullScreenflag:bool):boolean stdcall;
var
  Pixelformat: GLuint;            // Ukld formt pixel
  wc:TWndclass;                   // Struktura Windows Class
  dwExStyle:dword;                // Rozen styl okna
  dwStyle:dword;                  // Styl okna
  pfd: pixelformatdescriptor;     // Nastaven formtu pixel
  dmScreenSettings: Devmode;      // Md zazen
  h_Instance:hinst;               // Instance okna
  WindowRect: TRect;              // Obdlnk okna
  winx,winy: integer;             // Pozice okna ve win

begin
  WindowRect.Left := 0;                               // Nastav lev okraj na nulu
  WindowRect.Top := 0;                                // Nastav horn okraj na nulu
  WindowRect.Right := width;                          // Nastav prav okraj na zadanou hodnotu
  WindowRect.Bottom := height;                        // Nastav spodn okraj na zadanou hodnotu
  h_instance:=GetModuleHandle(nil);                   // Zsk instanci okna
  FullScreen:=FullScreenflag;                         // Nastav promnnou fullscreen na sprvnou hodnotu

  with wc do
    begin
      style:=CS_HREDRAW or CS_VREDRAW or CS_OWNDC;    // Pekreslen pi zmn velikosti a vlastn DC
      lpfnWndProc:=@WndProc;                          // Definuje proceduru okna
      cbClsExtra:=0;                                  // dn extra data
      cbWndExtra:=0;                                  // dn extra data
      hInstance:=h_Instance;                          // Instance
      hIcon:=LoadIcon(0,IDI_WINLOGO);                 // Standardn ikona
      hCursor:=LoadCursor(0,IDC_CROSS);               // kurzor s kkem
      hbrBackground:=0;                               // Pozad nen nutn
      lpszMenuName:=0;                                // Menu
      lpszClassName:='OpenGl';                        // Jmno tdy okna
    end;
  if  windows.RegisterClass(wc)=0 then                        // Registruje tdu okna
    begin
      MessageBox(0,'Failed To Register The Window Class.','Error',MB_OK or MB_ICONERROR);
      Result:=false;                                  // Pi chyb vrt false
      exit;
    end;
  if FullScreen then                                  // Budeme ve fullscreenu?
    begin
      ZeroMemory( @dmScreenSettings, sizeof(dmScreenSettings) );  // Vynulovn pamti
      with dmScreensettings do
        begin
          dmSize := sizeof(dmScreenSettings);         // Velikost struktury Devmode
          dmPelsWidth  := width;	                    // ka okna
	        dmPelsHeight := height;                     // Vka okna
          dmBitsPerPel := bits;                       // Barevn hloubka
          dmFields     := DM_BITSPERPEL or DM_PELSWIDTH or DM_PELSHEIGHT;
        end;
      // Pokus se pout prv definovan nastaven
      if (ChangeDisplaySettings(dmScreenSettings, CDS_FULLSCREEN))<>DISP_CHANGE_SUCCESSFUL THEN
        Begin
          // Nejde-li fullscreen, me uivatel spustit program v okn nebo ho opustit
          if MessageBox(0,'This FullScreen Mode Is Not Supported. Use Windowed Mode Instead?'
                                             ,'FullScreen Error',MB_YESNO or MB_ICONEXCLAMATION)= IDYES then
                FullScreen:=false                     // Bh v okn
          else
            begin
              // Zobraz uivateli zprvu, e program bude ukonen
              MessageBox(0,'Program Will Now Close.','Error',MB_OK or MB_ICONERROR);
              Result:=false;                          // Vrt FALSE
              exit;
            end;
          end;
    end;
  if FullScreen then                                  // Jsme stle ve fullscreenu?
    begin
      dwExStyle:=WS_EX_APPWINDOW;                     // Rozen styl okna
      dwStyle:=WS_POPUP or WS_CLIPSIBLINGS or WS_CLIPCHILDREN; // Styl okna
      Showcursor(false);                              // Skryje kurzor
    end
  else
    begin
      dwExStyle:=WS_EX_APPWINDOW or WS_EX_WINDOWEDGE;   // Rozen styl okna
      dwStyle := WS_MINIMIZEBOX or WS_CAPTION or WS_VISIBLE or WS_SYSMENU ; // Styl okna
    end;
  AdjustWindowRectEx(WindowRect,dwStyle,true,dwExStyle); // Pizpsoben velikosti okna
  winx:=trunc(GetSystemMetrics(SM_CXSCREEN)/2 - width/2);
  winy:=trunc(GetSystemMetrics(SM_CYSCREEN)/2 - (height + GetSystemMetrics (SM_CYMENU) + GetSystemMetrics (SM_CYCAPTION)) / 2);
    // MainMenu
  menu := TMainMenu.Create(nil);
  Item1 := TMenuItem.Create(nil);
  Item2 := TMenuItem.Create(nil);
  Item11:= TMenuItem.Create(nil);
  Item12:= TMenuItem.Create(nil);
  Item13:= TMenuItem.Create(nil);
  Item14:= TMenuItem.Create(nil);
  Item21:= TMenuItem.Create(nil);
  Item22:= TMenuItem.Create(nil);
  Item23:= TMenuItem.Create(nil);
  Item24:= TMenuItem.Create(nil);
  Item25:= TMenuItem.Create(nil);
  Item1.Caption := 'Akce';
  Item2.Caption := 'Nastaven';
  Item11.Caption := 'RRT';
  Item12.Caption := 'IGPPR';
  Item13.Caption := 'ISSD';
  Item14.Caption := 'Konec';
  Item21.Caption := 'Nastavit RRT';
  Item22.Caption := 'Nastavit IGPPR';
  Item23.Caption := 'Nastavit ISSD';
  Item24.Caption := 'Ostatn';
  Item25.Caption := 'Nast pekky';
  Menu.Items.Add(Item1);
  Menu.Items.Add(Item2);
  Item1.Add(Item11);
  Item1.Add(Item12);
  Item1.Add(Item13);
  Item1.Add(Item14);
  Item2.Add(Item21);
  Item2.Add(Item22);
  Item2.Add(Item23);
  Item2.Add(Item24);
  Item2.Add(Item25);
    // Vytvoen okna
  H_wnd:=CreateWindowEx(dwExStyle,                    // Rozen styl
                               'OpenGl',              // Jmno tdy
                               Title,                 // Titulek
                               dwStyle,               // Definovan styl
                               winx,winy,             // Pozice
                               WindowRect.Right-WindowRect.Left,  // Vpoet ky
                               WindowRect.Bottom-WindowRect.Top,  // Vpoet vky
                               0,                     // dn rodiovsk okno
                               menu.Handle,           // Menu
                               hinstance,             // Instance
                               nil);                  // Nepedat nic do WM_CREATE
  if h_Wnd=0 then                                     // Pokud se okno nepodailo vytvoit
    begin
      KillGlWindow();                                 // Zru okno
      MessageBox(0,'Window creation error.','Error',MB_OK or MB_ICONEXCLAMATION);
      Result:=false;                                  // Vrt chybu
      exit;
    end;
  with pfd do                                         // Oznmme Windows jak chceme ve nastavit
    begin
      nSize:= SizeOf( PIXELFORMATDESCRIPTOR );        // Velikost struktury
      nVersion:= 1;                                   // slo verze
      dwFlags:= PFD_DRAW_TO_WINDOW                    // Podpora okna
        or PFD_SUPPORT_OPENGL                         // Podpora OpenGL
        or PFD_DOUBLEBUFFER;                          // Podpora Double Bufferingu
      iPixelType:= PFD_TYPE_RGBA;                     // RGBA Format
      cColorBits:= bits;                              // Zvol barevnou hloubku
      cRedBits:= 0;                                   // Bity barev ignorovny
      cRedShift:= 0;
      cGreenBits:= 0;
      cBlueBits:= 0;
      cBlueShift:= 0;
      cAlphaBits:= 0;                                 // dn alpha buffer
      cAlphaShift:= 0;                                // Ignorovn Shift bit
      cAccumBits:= 0;                                 // dn akumulan buffer
      cAccumRedBits:= 0;                              // Akumulan bity ignorovny
      cAccumGreenBits:= 0;
      cAccumBlueBits:= 0;
      cAccumAlphaBits:= 0;
      cDepthBits:= 16;                                // 16-bitov hloubkov buffer (Z-Buffer)
      cStencilBits:= 0;                               // dn Stencil Buffer
      cAuxBuffers:= 0;                                // dn Auxiliary Buffer
      iLayerType:= PFD_MAIN_PLANE;                    // Hlavn vykreslovac vrstva
      bReserved:= 0;                                  // Rezervovno
      dwLayerMask:= 0;                                // Maska vrstvy ignorovna
      dwVisibleMask:= 0;
      dwDamageMask:= 0;
    end;
  h_Dc := GetDC(h_Wnd);                               // Zkus pipojit kontext zazen
  if h_Dc=0 then                                      // Podailo se pipojit kontext zazen?
    begin
      KillGLWindow();                                 // Zave okno
      MessageBox(0,'Cant''t create a GL device context.','Error',MB_OK or MB_ICONEXCLAMATION);
      Result:=false;                                  // Ukon program
      exit;
    end;
  PixelFormat := ChoosePixelFormat(h_Dc, @pfd);       // Zkus najt Pixel Format
  if (PixelFormat=0) then                             // Podailo se najt Pixel Format?
    begin
      KillGLWindow();                                 // Zave okno
      MessageBox(0,'Cant''t Find A Suitable PixelFormat.','Error',MB_OK or MB_ICONEXCLAMATION);
      Result:=false;                                  // Ukon program
      exit;
    end;
  if (not SetPixelFormat(h_Dc,PixelFormat,@pfd)) then  // Podailo se nastavit Pixel Format?
    begin
      KillGLWindow();                                 // Zave okno
      MessageBox(0,'Cant''t set PixelFormat.','Error',MB_OK or MB_ICONEXCLAMATION);
      Result:=false;                                  // Ukon program
      exit;
    end;
  h_Rc := wglCreateContext(h_Dc);                     // Podailo se vytvoit Rendering Context?
  if (h_Rc=0) then
    begin
      KillGLWindow();                                 // Zave okno
      MessageBox(0,'Cant''t create a GL rendering context.','Error',MB_OK or MB_ICONEXCLAMATION);
      Result:=false;                                  // Ukon program
      exit;
    end;
  if (not wglMakeCurrent(h_Dc, h_Rc)) then            // Podailo se aktivovat Rendering Context?
    begin
      KillGLWindow();                                 // Zave okno
      MessageBox(0,'Cant''t activate the GL rendering context.','Error',MB_OK or MB_ICONEXCLAMATION);
      Result:=false;                                  // Ukon program
      exit;
    end;
  ShowWindow(h_Wnd,SW_SHOW);                          // Zobrazen okna
  SetForegroundWindow(h_Wnd);                         // Do poped
  SetFOcus(h_Wnd);                                    // Zam fokus
  ReSizeGLScene(width,height);                        // Nastaven perspektivy OpenGL scny
  if (not InitGl()) then                              // Inicializace okna
    begin
      KillGLWindow();                                 // Zave okno
      MessageBox(0,'initialization failed.','Error',MB_OK or MB_ICONEXCLAMATION);
      Result:=false;                                  // Ukon program
      exit;
    end;
  Result:=true;                                       // Ve probhlo v podku
end;


function WinMain(hInstance: HINST;                    // Instance
		 hPrevInstance: HINST;                            // Pedchoz instance
		 lpCmdLine: PChar;                                // Parametry pkazov dky
		 nCmdShow: integer):                              // Stav zobrazen okna
     integer; stdcall;
var
  msg: TMsg;                                          // Struktura zprv systmu
  done: Bool;                                         // Promnn pro ukonen programu
begin
  done:=false;
  // Dotaz na uivatele pro fullscreen/okno
  {if MessageBox(0,'Would You Like To Run In FullScreen Mode?','Start FullScreen',
                             MB_YESNO or MB_ICONQUESTION)=IDNO then
    FullScreen:=false                                 // Bh v okn
  else
    FullScreen:=true;}                                 // Fullscreen
  InitializeCriticalSection(cs);
	InitializeCriticalSection(cs_CARS);
  InitializeCriticalSection(CS_cell);
  InitializeCriticalSection(CS_obs);
  FullScreen:=false;
  if not CreateGLWindow('# Path Planning , Sindelar [2010]',795,595,16,FullScreen) then // Vytvoen OpenGL okna
    begin
      Result := 0;                                    // Konec programu pi chyb
      exit;
    end;
  while not done do                                   // Hlavn cyklus programu
    begin
      if (PeekMessage(msg, 0, 0, 0, PM_REMOVE)) then  // Pila zprva?
        begin
          if msg.message=WM_QUIT then                 // Obdreli jsme zprvu pro ukonen?
            done:=true                                // Konec programu
          else
            begin
	          TranslateMessage(msg);                    // Pelo zprvu
	          DispatchMessage(msg);                     // Odele zprvu
	        end;
        end
      else      // Pokud nedola dn zprva
        begin
            // Je program aktivn, ale nelze kreslit? Byl stisknut ESC?
          if (active and not(DrawGLScene()) or keys[VK_ESCAPE]) then
            done:=true                                // Ukonme program
          else                                        // Pekreslen scny
            SwapBuffers(h_Dc);                        // Prohozen buffer (Double Buffering)
          if keys[VK_F1] then                         // Byla stisknuta klvesa F1?
            begin
            Keys[VK_F1] := false;                     // Ozna ji jako nestisknutou
            KillGLWindow();                           // Zru okno
            FullScreen := not FullScreen;             // Negace fullscreen
            // Znovuvytvoen okna
            if not CreateGLWindow('# Path Planning , Sindelar [2009]',640,480,16,fullscreen) then
              Result := 0;                            // Konec programu pokud nebylo vytvoeno
            end;
        end;
    end;                                              // Konec smyky while
  killGLwindow();                                     // Zave okno
  DeleteCriticalSection(cs);
	DeleteCriticalSection(cs_CARS);
  DeleteCriticalSection(CS_cell);
  DeleteCriticalSection(CS_obs);
  result:=msg.wParam;                                 // Ukonen programu
end;



//------------------------------------------------------------------------------------------
// RRT
//------------------------------------------------------------------------------------------

begin
  start:=vector_q.create(0,120,210,0);
  goal:=vector_q.create(570,210,0);
  q:=vector_q.create(start.x,start.y,0);
  u:=vector_u.create(0,0,0,0);
  SetParam;
  fObstacles:=TObstacles.Create;
  fObstacles.Load(ExtractFilePath(Application.ExeName)+'data.obs');

  TreeRRT:=TRRT.Create(MaxX,MaxY);
  TreeRRT.SetMethod(MaxX,MaxY,RRT_Iter,EndD);
  TreeRRT.SetObstacles(fObstacles.Obstacles);

  IGPPR:=TIGPPR.Create(MaxX,MaxY,goal);
  IGPPR.SetObstacles(fObstacles.Obstacles);

  ISSD:=TISSD.Create(MaxX,MaxY,goal);
  ISSD.SetObstacles(fObstacles.Obstacles);

  Close:=True;
  OKRightDlg:=TOKRightDlg.Create(nil); // vytvoreni dialog.okna ProgressBaru
  WinMain( hInstance, hPrevInst, CmdLine, CmdShow );
end.
